"""Test models."""
from django.core.exceptions import ValidationError
from django.test import SimpleTestCase
from djmoney.money import Money

from django_pain.models import BankAccount, BankPayment


class TestBankPayment(SimpleTestCase):
    """Test BankPayment model."""

    def test_constraint_success(self):
        """Test clean method with not violated constraint."""
        account = BankAccount(account_number='123', currency='USD')
        payment = BankPayment(identifier='PAYMENT', account=account, amount=Money('999.00', 'USD'))
        payment.clean()

    def test_constraint_error(self):
        """Test clean method with violated constraint."""
        account = BankAccount(account_number='123', currency='USD')
        payment = BankPayment(identifier='PAYMENT', account=account, amount=Money('999.00', 'CZK'))
        with self.assertRaises(ValidationError, msg='Bank payment PAYMENT is in different currency (CZK) '
                                                    'than bank account 123 (USD).'):
            payment.clean()



"""Payments and invoices models."""
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import gettext_lazy as _
from djmoney.models.fields import CurrencyField, MoneyField

from django_pain.constants import (CURRENCY_PRECISION, PAYMENT_STATE_EXPORTED, PAYMENT_STATE_IMPORTED,
                                   PAYMENT_STATE_PROCESSED)

PAYMENT_STATE_CHOICES = (
    (PAYMENT_STATE_IMPORTED, _('imported')),
    (PAYMENT_STATE_PROCESSED, _('processed')),
    (PAYMENT_STATE_EXPORTED, _('exported')),
)


class BankAccount(models.Model):
    """Bank account."""

    account_number = models.TextField(verbose_name=_('Account number'))
    account_name = models.TextField(blank=True, verbose_name=_('Account name'))
    currency = CurrencyField()


class BankPayment(models.Model):
    """Bank payment."""

    identifier = models.TextField(verbose_name=_('Payment ID'))
    account = models.ForeignKey(BankAccount, on_delete=models.CASCADE)
    create_time = models.DateTimeField(auto_now_add=True, verbose_name=_('Create time'))
    transaction_date = models.DateField(verbose_name=_('Transaction date'))

    counter_account_number = models.TextField(verbose_name=_('Counter account number'))
    counter_account_name = models.TextField(blank=True, verbose_name=_('Counter account name'))

    amount = MoneyField(max_digits=64, decimal_places=CURRENCY_PRECISION, verbose_name=_('Amount'))
    description = models.TextField(blank=True, verbose_name=_('Description'))
    state = models.IntegerField(choices=PAYMENT_STATE_CHOICES, default=PAYMENT_STATE_IMPORTED,
                                verbose_name=_('Payment state'))

    class Meta:
        """Model Meta class."""

        unique_together = ('identifier', 'account')

    def clean(self):
        """Check whether payment currency is the same as currency of related bank account."""
        if self.account.currency != self.amount.currency.code:
            raise ValidationError('Bank payment {} is in different currency ({}) than bank account {} ({}).'.format(
                self.identifier, self.amount.currency.code, self.account.account_number, self.account.currency
            ))
        super().clean()



"""Application-wide constants."""

# Number of decimal places of currency amounts.
# Bitcoin has 8, so 10 should be enough for most practical purposes.
CURRENCY_PRECISION = 10

# Payment states constants.
PAYMENT_STATE_IMPORTED = 0
PAYMENT_STATE_PROCESSED = 1
PAYMENT_STATE_EXPORTED = 2



"""Django app configuration."""
from django.apps import AppConfig


class DjangoPainConfig(AppConfig):
    """Configuration of django_pain app."""

    name = 'django_pain'
    verbose_name = 'Django Payments and Invoices'



#!/usr/bin/python3
"""Setup script for django_pain."""
from setuptools import find_packages, setup

import django_pain

INSTALL_REQUIRES = ['Django>=2.0', 'django-money']
EXTRAS_REQUIRE = {'quality': ['isort', 'flake8', 'pydocstyle', 'polint']}

setup(name='django-pain',
      version=django_pain.__version__,
      description='Django application for managing bank payments and invoices',
      url='https://github.com/stinovlas/django-pain',
      author='Jan Musílek',
      author_email='jan.musilek@nic.cz',
      packages=find_packages(),
      install_requires=INSTALL_REQUIRES,
      extras_require=EXTRAS_REQUIRE,
      classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
      ])



"""Payment symbols."""
from django.db import models
from django.utils.translation import gettext_lazy as _

from .bank import BankPayment


class PaymentSymbols(models.Model):
    """Payment symbols (specific for Czech Republic and Slovak Republic)."""

    payment = models.OneToOneField(BankPayment, on_delete=models.CASCADE, related_name='symbols')

    constant_symbol = models.CharField(max_length=10, blank=True, verbose_name=_('Constant symbol'))
    variable_symbol = models.CharField(max_length=10, blank=True, verbose_name=_('Variable symbol'))
    specific_symbol = models.CharField(max_length=10, blank=True, verbose_name=_('Specific symbol'))
